@@ -95,7 +95,7 @@ class PuppetLint
 
     # Check if the input is an SE Linux policy package file (which also use
     # the .pp extension), which all have the first 4 bytes 0xf97cff8f.
-    @code = '' if @code[0..3].unpack('V').first == 0xf97cff8f
+    @code = '' if @code[0..3].unpack1('V') == 0xf97cff8f
   end
 
   # Internal: Retrieve the format string to be used when writing problems to
@@ -124,6 +124,7 @@ class PuppetLint
     puts format % message
 
     puts "  #{message[:reason]}" if message[:kind] == :ignored && !message[:reason].nil?
+    print_context(message)
   end
 
   # Internal: Format a problem message and print it to STDOUT so GitHub Actions
@@ -154,7 +155,10 @@ class PuppetLint
   def print_context(message)
     return if message[:check] == 'documentation'
     return if message[:kind] == :fixed
-    line = get_context(message)
+
+    line = message[:context]
+    return unless line
+
     offset = line.index(%r{\S}) || 1
     puts "\n  #{line.strip}"
     printf("%#{message[:column] + 2 - offset}s\n\n", '^')
@@ -168,6 +172,8 @@ class PuppetLint
   # Returns array of problem.
   def report(problems)
     json = []
+    print_stdout = !(configuration.json || configuration.sarif)
+
     problems.each do |message|
       next if message[:kind] == :ignored && !PuppetLint.configuration.show_ignored
 
@@ -175,16 +181,16 @@ class PuppetLint
 
       next unless message[:kind] == :fixed || [message[:kind], :all].include?(configuration.error_level)
 
-      if configuration.json || configuration.sarif
-        message['context'] = get_context(message) if configuration.with_context
-        json << message
-      else
+      message[:context] = get_context(message) if configuration.with_context
+
+      json << message
+
+      if print_stdout
         format_message(message)
-        print_context(message) if configuration.with_context
         print_github_annotation(message) if configuration.github_actions
       end
     end
-    $stderr.puts 'Try running `puppet parser validate <file>`' if problems.any? { |p| p[:check] == :syntax }
+    warn 'Try running `puppet parser validate <file>`' if problems.any? { |p| p[:check] == :syntax }
     json
   end
 
@@ -225,7 +231,7 @@ class PuppetLint
 
   # Public: Print any problems that were found out to stdout.
   #
-  # Returns nothing.
+  # Returns an array of problems.
   def print_problems
     report(@problems)
   end
@@ -246,8 +252,8 @@ class PuppetLint
   #   end
   def self.new_check(name, &block)
     class_name = name.to_s.split('_').map(&:capitalize).join
-    klass = PuppetLint.const_set("Check#{class_name}", Class.new(PuppetLint::CheckPlugin))
-    klass.const_set('NAME', name)
+    klass = PuppetLint.const_set(:"Check#{class_name}", Class.new(PuppetLint::CheckPlugin))
+    klass.const_set(:NAME, name)
     klass.class_exec(&block)
     PuppetLint.configuration.add_check(name, klass)
     PuppetLint::Data.ignore_overrides[name] ||= {}
