@@ -1,24 +1,30 @@
 require 'singleton'
 require 'set'
 
-# Public: A singleton class storing all the information about the manifest
-# being analysed.
+# A singleton class storing all the information about the manifest being
+# analysed.
+#
+# @api public
 class PuppetLint::Data
   include Singleton
 
   class << self
-    # Internal: Get/Set the full expanded path to the manifest file being
-    # checked.
+    # Get/Set the full expanded path to the manifest file being checked.
+    #
+    # @api private
     attr_reader :path, :fullpath, :filename
 
-    # Internal: Get/Set the raw manifest data, split by \n.
+    # Get/Set the raw manifest data, split by \n.
+    #
+    # @api private
     attr_accessor :manifest_lines
 
-    # Internal: Store the tokenised manifest.
+    # Store the tokenised manifest.
     #
-    # tokens - The Array of PuppetLint::Lexer::Token objects to store.
+    # @param [Array[PuppetLint::Lexer::Token]] tokens The Array of PuppetLint::Lexer::Token objects to store.
+    # @return [nil]
     #
-    # Returns nothing.
+    # @api private
     def tokens=(tokens)
       @tokens = tokens
       @title_tokens = nil
@@ -32,14 +38,17 @@ class PuppetLint::Data
       @defaults_indexes = nil
     end
 
+    # @api private
     def ruby1?
       @ruby1 = RbConfig::CONFIG['MAJOR'] == '1' if @ruby1.nil?
       @ruby1
     end
 
-    # Public: Get the tokenised manifest.
+    # Get the tokenised manifest.
     #
-    # Returns an Array of PuppetLint::Lexer::Token objects.
+    # @return [Array[PuppetLint::Lexer::Token]]
+    #
+    # @api public
     def tokens
       calling_method = if ruby1?
                          begin
@@ -62,7 +71,12 @@ class PuppetLint::Data
       end
     end
 
-    # Public: Add new token.
+    # Add new token.
+    #
+    # @param [Integer] index
+    # @param [PuppetLint::Lexer::Token] token
+    #
+    # @api public
     def insert(index, token)
       current_token = tokens[index - 1]
       token.next_token = current_token.next_token
@@ -72,7 +86,7 @@ class PuppetLint::Data
 
       unless formatting_tokens.include?(token.type)
         current_token.next_token.prev_code_token = token unless current_token.next_token.nil?
-        next_nf_idx = tokens[index..-1].index { |r| !formatting_tokens.include?(r.type) }
+        next_nf_idx = tokens[index..].index { |r| !formatting_tokens.include?(r.type) }
         unless next_nf_idx.nil?
           next_nf_token = tokens[index + next_nf_idx]
           token.next_code_token = next_nf_token
@@ -92,14 +106,16 @@ class PuppetLint::Data
       end
 
       current_token.next_token = token
-      unless formatting_tokens.include?(token.type)
-        current_token.next_code_token = token
-      end
+      current_token.next_code_token = token unless formatting_tokens.include?(token.type)
 
       tokens.insert(index, token)
     end
 
-    # Public: Removes a token
+    # Remove a token
+    #
+    # @param [PuppetLint::Lexer::Token] token
+    #
+    # @api public
     def delete(token)
       token.next_token.prev_token = token.prev_token unless token.next_token.nil?
       token.prev_token.next_token = token.next_token unless token.prev_token.nil?
@@ -111,26 +127,29 @@ class PuppetLint::Data
       tokens.delete(token)
     end
 
-    # Internal: Store the path to the manifest file and populate fullpath and
-    # filename.
+    # Store the path to the manifest file and populate fullpath and filename.
     #
-    # val - The path to the file as a String.
+    # @param [String] val The path to the file
     #
-    # Returns nothing.
+    # @return [nil]
+    #
+    # @api private
     def path=(val)
       @path = val
       if val.nil?
         @fullpath = nil
         @filename = nil
       else
-        @fullpath = File.expand_path(val, ENV['PWD'])
+        @fullpath = File.expand_path(val, ENV.fetch('PWD', nil))
         @filename = File.basename(val)
       end
     end
 
-    # Internal: Retrieve a list of tokens that represent resource titles
+    # Retrieve a list of tokens that represent resource titles
     #
-    # Returns an Array of PuppetLint::Lexer::Token objects.
+    # @return [Array[PuppetLint::Lexer::Token]]
+    #
+    # @api private
     def title_tokens
       @title_tokens ||= begin
         result = []
@@ -155,28 +174,44 @@ class PuppetLint::Data
       end
     end
 
-    # Internal: Calculate the positions of all resource declarations within the
+    # Determine if the given token contains a CLASSREF in the token chain..
+    #
+    # @param [PuppetLint::Lexer::Token] token
+    #
+    # @api private
+    def classref?(token)
+      current_token = token
+      while (current_token = current_token.prev_code_token)
+        return true if current_token.type == :CLASSREF
+        return false if current_token.type == :NAME
+      end
+    end
+
+    # Calculate the positions of all resource declarations within the
     # tokenised manifest. These positions only point to the content of the
     # resource declarations, they do not include resource types or titles.
     #
-    # Returns an Array of Hashes, each containing:
-    #   :start - An Integer position in the `tokens` Array pointing to the
-    #            first Token of a resource declaration.
-    #   :end   - An Integer position in the `tokens` Array pointing to the last
-    #            Token of a resource declaration.
+    # @return [Array[Hash[Symbol, Integer]]] each hash contains :start and
+    #   :end pointing to the first and last {PuppetLint::Lexer::Token} of a
+    #   resource declaration
+    #
+    # @api private
     def resource_indexes
       @resource_indexes ||= begin
         marker = 0
         result = []
         tokens.select { |t| t.type == :COLON }.each do |colon_token|
           next unless colon_token.next_code_token && colon_token.next_code_token.type != :LBRACE
+          next if classref?(colon_token)
 
-          rel_start_idx = tokens[marker..-1].index(colon_token)
+          rel_start_idx = tokens[marker..].index(colon_token)
           break if rel_start_idx.nil?
+
           start_idx = rel_start_idx + marker
           end_token = colon_token.next_token_of([:SEMIC, :RBRACE])
-          rel_end_idx = tokens[start_idx..-1].index(end_token)
+          rel_end_idx = tokens[start_idx..].index(end_token)
           raise PuppetLint::SyntaxError, colon_token if rel_end_idx.nil?
+
           marker = rel_end_idx + start_idx
 
           result << {
@@ -184,19 +219,22 @@ class PuppetLint::Data
             end: marker,
             tokens: tokens[start_idx..marker],
             type: find_resource_type_token(start_idx),
-            param_tokens: find_resource_param_tokens(tokens[start_idx..marker]),
+            param_tokens: find_resource_param_tokens(tokens[start_idx..marker])
           }
         end
         result
       end
     end
 
-    # Internal: Find the Token representing the type of a resource definition.
+    # Find the Token representing the type of a resource definition.
     #
-    # index - The Integer pointing to the start of the resource in the `tokens`
-    #         array.
+    # @param [Integer] index
+    #   The Integer pointing to the start of the resource in the `tokens`
+    #   array.
     #
-    # Returns a Token object.
+    # @return [PuppetLint::Lexer::Token]
+    #
+    # @api private
     def find_resource_type_token(index)
       lbrace_idx = tokens[0..index].rindex do |token|
         token.type == :LBRACE && token.prev_code_token.type != :QMARK
@@ -207,13 +245,15 @@ class PuppetLint::Data
       tokens[lbrace_idx].prev_code_token
     end
 
-    # Internal: Find all the Token objects representing the parameter names in
+    # Find all the Token objects representing the parameter names in
     # a resource definition.
     #
-    # resource_tokens - An Array of Token objects that comprise the resource
-    #                   definition.
+    # @param [Array[PuppetLint::Lexer::Token]] resource_tokens
+    #   An Array of Token objects that comprise the resource definition.
     #
-    # Returns an Array of Token objects.
+    # @return [Array[PuppetLint::Lexer::Token]]
+    #
+    # @api private
     def find_resource_param_tokens(resource_tokens)
       param_tokens = []
 
@@ -224,16 +264,14 @@ class PuppetLint::Data
 
         break unless resource_tokens.include?(iter_token)
 
-        if iter_token && iter_token.next_code_token.type == :FARROW
-          param_tokens << iter_token
-        end
+        param_tokens << iter_token if iter_token && iter_token.next_code_token.type == :FARROW
       end
 
       param_tokens
     end
 
-    # Internal: Calculate the positions of all class definitions within the
-    # `tokens` Array.
+    # Calculate the positions of all class definitions within the `tokens`
+    # Array.
     #
     # Returns an Array of Hashes, each containing:
     #   :start  - An Integer position in the `tokens` Array pointing to the
@@ -242,25 +280,13 @@ class PuppetLint::Data
     #             Token of a class definition.
     #   :tokens - An Array consisting of all the Token objects that make up the
     #             class definition.
+    #
+    # @api private
     def class_indexes
       @class_indexes ||= definition_indexes(:CLASS)
     end
 
-    # Internal: Calculate the positions of all defined type definitions within
-    # the `tokens` Array.
-    #
-    # Returns an Array of Hashes, each containing:
-    #   :start  - An Integer position in the `tokens` Array pointing to the
-    #             first Token of a defined type definition.
-    #   :end    - An Integer position in the `tokens` Array pointing to the last
-    #             Token of a defined type definition.
-    #   :tokens - An Array consisting of all the Token objects that make up the
-    #             defined type.
-    def defined_type_indexes
-      @defined_type_indexes ||= definition_indexes(:DEFINE)
-    end
-
-    # Internal: Calculate the positions of all node definitions within the
+    # Calculate the positions of all defined type definitions within the
     # `tokens` Array.
     #
     # Returns an Array of Hashes, each containing:
@@ -270,12 +296,30 @@ class PuppetLint::Data
     #             Token of a defined type definition.
     #   :tokens - An Array consisting of all the Token objects that make up the
     #             defined type.
+    #
+    # @api private
+    def defined_type_indexes
+      @defined_type_indexes ||= definition_indexes(:DEFINE)
+    end
+
+    # Calculate the positions of all node definitions within the `tokens`
+    # Array.
+    #
+    # Returns an Array of Hashes, each containing:
+    #   :start  - An Integer position in the `tokens` Array pointing to the
+    #             first Token of a defined type definition.
+    #   :end    - An Integer position in the `tokens` Array pointing to the last
+    #             Token of a defined type definition.
+    #   :tokens - An Array consisting of all the Token objects that make up the
+    #             defined type.
+    #
+    # @api private
     def node_indexes
       @node_indexes ||= definition_indexes(:NODE)
     end
 
-    # Internal: Calculate the positions of all the specified defintion types
-    # within the `tokens` Array.
+    # Calculate the positions of all the specified defintion types within the
+    # `tokens` Array.
     #
     # Returns an Array of Hashes, each containing:
     #   :start  - An Integer position in the `tokens` Array pointing to the
@@ -284,6 +328,8 @@ class PuppetLint::Data
     #             Token of a definition.
     #   :tokens - An Array consisting of all the Token objects that make up the
     #             definition.
+    #
+    # @api private
     def definition_indexes(type)
       result = []
       tokens.each_with_index do |token, i|
@@ -293,7 +339,7 @@ class PuppetLint::Data
         paren_depth = 0
         in_params = false
         inherited_class = nil
-        tokens[i + 1..-1].each_with_index do |definition_token, j|
+        tokens[i + 1..].each_with_index do |definition_token, j|
           case definition_token.type
           when :INHERITS
             inherited_class = definition_token.next_code_token
@@ -307,19 +353,17 @@ class PuppetLint::Data
             brace_depth += 1
           when :RBRACE
             brace_depth -= 1
-            if brace_depth.zero? && !in_params
-              if token.next_code_token.type != :LBRACE
-                result << {
-                  start: i,
-                  end: i + j + 1,
-                  tokens: tokens[i..(i + j + 1)],
-                  param_tokens: param_tokens(tokens[i..(i + j + 1)]),
-                  type: type,
-                  name_token: token.next_code_token,
-                  inherited_token: inherited_class,
-                }
-                break
-              end
+            if brace_depth.zero? && !in_params && (token.next_code_token.type != :LBRACE)
+              result << {
+                start: i,
+                end: i + j + 1,
+                tokens: tokens[i..(i + j + 1)],
+                param_tokens: param_tokens(tokens[i..(i + j + 1)]),
+                type: type,
+                name_token: token.next_code_token,
+                inherited_token: inherited_class
+              }
+              break
             end
           end
         end
@@ -327,8 +371,7 @@ class PuppetLint::Data
       result
     end
 
-    # Internal: Calculate the positions of all function calls within
-    # `tokens` Array.
+    # Calculate the positions of all function calls within `tokens` Array.
     #
     # Returns an Array of Hashes, each containing:
     #   :start  - An Integer position in the `tokens` Array pointing to the
@@ -337,11 +380,13 @@ class PuppetLint::Data
     #             Token of a function call
     #   :tokens - An Array consisting of all the Token objects that make up the
     #             function call.
+    #
+    # @api private
     def function_indexes
       @function_indexes ||= begin
         functions = []
         tokens.each_with_index do |token, token_idx|
-          next unless token.type == :NAME
+          next unless token.type == :FUNCTION_NAME
           next unless token_idx.zero? ||
                       (token_idx == 1 && tokens[0].type == :WHITESPACE) ||
                       [:NEWLINE, :INDENT].include?(token.prev_token.type) ||
@@ -354,7 +399,7 @@ class PuppetLint::Data
           level = 0
           real_idx = 0
           in_paren = false
-          tokens[token_idx + 1..-1].each_with_index do |cur_token, cur_token_idx|
+          tokens[token_idx + 1..].each_with_index do |cur_token, cur_token_idx|
             break if level.zero? && in_paren
             break if level.zero? && cur_token.type == :NEWLINE
 
@@ -369,15 +414,14 @@ class PuppetLint::Data
           functions << {
             start: token_idx,
             end: real_idx,
-            tokens: tokens[token_idx..real_idx],
+            tokens: tokens[token_idx..real_idx]
           }
         end
         functions
       end
     end
 
-    # Internal: Calculate the positions of all array values within
-    # `tokens` Array.
+    # Calculate the positions of all array values within `tokens` Array.
     #
     # Returns an Array of Hashes, each containing:
     #   :start  - An Integer position in the `tokens` Array pointing to the
@@ -386,6 +430,8 @@ class PuppetLint::Data
     #             Token of an array value
     #   :tokens - An Array consisting of all the Token objects that make up the
     #             array value.
+    #
+    # @api private
     def array_indexes
       @array_indexes ||= begin
         arrays = []
@@ -393,7 +439,7 @@ class PuppetLint::Data
           next unless token.type == :LBRACK
 
           real_idx = 0
-          tokens[token_idx + 1..-1].each_with_index do |cur_token, cur_token_idx|
+          tokens[token_idx + 1..].each_with_index do |cur_token, cur_token_idx|
             real_idx = token_idx + 1 + cur_token_idx
             break if cur_token.type == :RBRACK
           end
@@ -405,15 +451,14 @@ class PuppetLint::Data
           arrays << {
             start: token_idx,
             end: real_idx,
-            tokens: tokens[token_idx..real_idx],
+            tokens: tokens[token_idx..real_idx]
           }
         end
         arrays
       end
     end
 
-    # Internal: Calculate the positions of all hash values within
-    # `tokens` Array.
+    # Calculate the positions of all hash values within `tokens` Array.
     #
     # Returns an Array of Hashes, each containing:
     #   :start  - An Integer position in the `tokens` Array pointing to the
@@ -422,6 +467,8 @@ class PuppetLint::Data
     #             Token of an hash value
     #   :tokens - An Array consisting of all the Token objects that make up the
     #             hash value.
+    #
+    # @api private
     def hash_indexes
       @hash_indexes ||= begin
         hashes = []
@@ -432,25 +479,25 @@ class PuppetLint::Data
 
           level = 0
           real_idx = 0
-          tokens[token_idx + 1..-1].each_with_index do |cur_token, cur_token_idx|
+          tokens[token_idx + 1..].each_with_index do |cur_token, cur_token_idx|
             real_idx = token_idx + 1 + cur_token_idx
 
             level += 1 if cur_token.type == :LBRACE
             level -= 1 if cur_token.type == :RBRACE
-            break if level < 0
+            break if level.negative?
           end
 
           hashes << {
             start: token_idx,
             end: real_idx,
-            tokens: tokens[token_idx..real_idx],
+            tokens: tokens[token_idx..real_idx]
           }
         end
         hashes
       end
     end
 
-    # Internal: Calculate the positions of all defaults declarations within
+    # Calculate the positions of all defaults declarations within
     # `tokens` Array.
     #
     # Returns an Array of Hashes, each containing:
@@ -460,6 +507,8 @@ class PuppetLint::Data
     #             Token of the defaults declaration
     #   :tokens - An Array consisting of all the Token objects that make up the
     #             defaults declaration.
+    #
+    # @api private
     def defaults_indexes
       @defaults_indexes ||= begin
         defaults = []
@@ -469,7 +518,7 @@ class PuppetLint::Data
 
           real_idx = 0
 
-          tokens[token_idx + 1..-1].each_with_index do |cur_token, cur_token_idx|
+          tokens[token_idx + 1..].each_with_index do |cur_token, cur_token_idx|
             real_idx = token_idx + 1 + cur_token_idx
             break if cur_token.type == :RBRACE
           end
@@ -477,21 +526,24 @@ class PuppetLint::Data
           defaults << {
             start: token_idx,
             end: real_idx,
-            tokens: tokens[token_idx..real_idx],
+            tokens: tokens[token_idx..real_idx]
           }
         end
         defaults
       end
     end
 
-    # Internal: Finds all the tokens that make up the defined type or class
+    # Finds all the tokens that make up the defined type or class
     # definition parameters.
     #
-    # these_tokens - An Array of PuppetLint::Lexer::Token objects that make up
-    #                the defined type or class definition.
+    # @param [Array[PuppetLint::Lexer::Token]] these_tokens
+    #   An Array of PuppetLint::Lexer::Token objects that make up the defined
+    #   type or class definition.
     #
-    # Returns an Array of PuppetLint::Lexer::Token objects or nil if it takes
-    # no parameters.
+    # @return [Array[PuppetLint::Lexer::Token], nil] objects or nil if it takes
+    #   no parameters.
+    #
+    # @api private
     def param_tokens(these_tokens)
       depth = 0
       lparen_idx = nil
@@ -520,28 +572,34 @@ class PuppetLint::Data
       end
     end
 
-    # Internal: Retrieves a list of token types that are considered to be
+    # Retrieves a list of token types that are considered to be
     # formatting tokens (whitespace, newlines, etc).
     #
-    # Returns an Array of Symbols.
+    # @return [Array[Symbol]]
+    #
+    # @api private
     def formatting_tokens
       @formatting_tokens ||= PuppetLint::Lexer::FORMATTING_TOKENS
     end
 
-    # Internal: Retrieves a Hash of Sets. Each key is a check name Symbol and
+    # Retrieves a Hash of Sets. Each key is a check name Symbol and
     # the Set of Integers returned lists all the lines that the check results
     # should be ignored on.
     #
-    # Returns a Hash of Sets of Integers.
+    # @return [Hash[Symbol, Set[Integer]]]
+    #
+    # @api private
     def ignore_overrides
       @ignore_overrides ||= {}
     end
 
-    # Internal: Parses all COMMENT, MLCOMMENT and SLASH_COMMENT tokens looking
+    # Parses all COMMENT, MLCOMMENT and SLASH_COMMENT tokens looking
     # for control comments (comments that enable or disable checks). Builds the
     # contents of the `ignore_overrides` hash.
     #
-    # Returns nothing.
+    # @return [nil]
+    #
+    # @api private
     def parse_control_comments
       @ignore_overrides.each_key { |check| @ignore_overrides[check].clear }
 
@@ -551,7 +609,7 @@ class PuppetLint::Data
         comment_token_types.include?(token.type)
       end
       control_comment_tokens = comment_tokens.select do |token|
-        token.value.strip =~ %r{\Alint\:(ignore\:[\w\d]+|endignore)}
+        token.value.strip =~ %r{\Alint:(ignore:[\w\d]+|endignore)}
       end
 
       stack = []
@@ -561,12 +619,12 @@ class PuppetLint::Data
 
         comment_words = token.value.strip.split(%r{\s+})
         comment_words.each_with_index do |word, i|
-          if %r{\Alint\:(ignore|endignore)}.match?(word)
+          if %r{\Alint:(ignore|endignore)}.match?(word)
             comment_data << word
           else
             # Once we reach the first non-controlcomment word, assume the rest
             # of the words are the reason.
-            reason = comment_words[i..-1]
+            reason = comment_words[i..]
             break
           end
         end
@@ -591,7 +649,7 @@ class PuppetLint::Data
             if top_override.nil?
               # TODO: refactor to provide a way to expose problems from
               # PuppetLint::Data via the normal problem reporting mechanism.
-              $stderr.puts "WARNING: lint:endignore comment with no opening lint:ignore:<check> comment found on line #{token.line}"
+              warn "WARNING: lint:endignore comment with no opening lint:ignore:<check> comment found on line #{token.line}"
             else
               top_override.each do |start|
                 next if start.nil?
@@ -607,7 +665,7 @@ class PuppetLint::Data
       end
 
       stack.each do |control|
-        $stderr.puts "WARNING: lint:ignore:#{control[0][2]} comment on line #{control[0][0]} with no closing lint:endignore comment"
+        warn "WARNING: lint:ignore:#{control[0][2]} comment on line #{control[0][0]} with no closing lint:endignore comment"
       end
     end
   end
