@@ -1,5 +1,3 @@
-# encoding: utf-8
-
 require 'spec_helper'
 
 describe PuppetLint::Lexer::StringSlurper do
@@ -452,13 +450,13 @@ describe PuppetLint::Lexer::StringSlurper do
   end
 
   describe '#consumed_chars' do
-    subject { described_class.new(string).tap(&:parse).consumed_chars }
+    subject(:consumed_chars) { described_class.new(string).tap(&:parse).consumed_chars }
 
     context 'when slurping a string containing multibyte characters' do
       let(:string) { 'accentués"' }
 
       it 'counts the multibyte character as a single consumed character' do
-        is_expected.to eq(10)
+        expect(consumed_chars).to eq(10)
       end
     end
 
@@ -466,7 +464,7 @@ describe PuppetLint::Lexer::StringSlurper do
       let(:string) { '"' }
 
       it 'consumes only the closing quote' do
-        is_expected.to eq(1)
+        expect(consumed_chars).to eq(1)
       end
     end
   end
